==> lemonade-0.0.6.tgz: package/kernel/adaptors/mysql.js <==
/*****************************************************************************
 * Lemonade JS
 * https://lemonadejs.com
 *
 * ----
 *
 * Mysql
 *
 *  Adaptor to connect and execute queries on a Mysql replication farm. Will
 *  create different connections depending on the first word in the query,
 *  directing modifying requests to the master and selects to the slave
 *
 *****************************************************************************/

/**
 * Module dependencies
 */
var adaptor = require('mysql');

/**
 * Mysql
 *
 * @param {string} db
 * @param {object} master
 * @param {object} slave
 * @constructor
 */
function Mysql(db, master, slave) {

    /**
     * Status object
     */
    this.status = {
        querycnt : {
             master : 0
            ,slave  : 0
        }
        ,running : {}
    };

    /**
     * Master pool
     * @type {*}
     * @private
     */
    this._masterpool = adaptor.createPool({
         host     : master.host
        ,port     : master.port
        ,user     : master.user
        ,password : master.pass
        ,database : db

        ,connectionLimit : 10
    });

    if (typeof slave !== 'undefined') {
        /**
         * Slave pool
         * @type {*}
         * @private
         */
        this._slavepool = adaptor.createPool({
             host     : slave.host
            ,port     : slave.port
            ,user     : slave.user
            ,password : slave.pass
            ,database : db
    
            ,connectionLimit : 10
        });
    } else {
        this._slavepool = this._masterpool;
        this.status.singlepool = true;
    }

    /**
     * Wait until both pools are connected, then emit
     * a general connect event
     */
    this._poolsnc = 2;
    this.kernel.on('_Mysql-pool-connect', function() {
        if (!--this._poolsnc) {
            this.kernel.emit('Mysql-connect');
        }
    }.bind(this));

    /**
     * Try to connect the pools
     */
    this.connect();
}

/**
 * Do a query on each pool (master and slave), on both
 * queries success' we have a "connected" adaptor
 *
 * @return
 */
Mysql.prototype.connect = function() {

    this._poolsnc = 2;
    var cb = function(error) {
        if (error) {
            this.kernel.emit('Mysql-connect-error', error);
        } else {
            this.kernel.emit('_Mysql-pool-connect');
        }

    }.bind(this);

    /**
     * Dummy select on the slave
     */
    this.query('SELECT 1', [], cb);

    /**
     * Dummy select on the master
     */
    this.query('/* Force master */ SELECT 1', [], cb);

};

/**
 * Fetches a connection from the pool and runs a query
 *
 * @param {string} query
 * @param {array} params
 * @param {function} callback
 */
Mysql.prototype.query = function(query, params, callback) {
    var
    pool = choosePool.call(this, query);
    pool.getConnection(function(err, conn) {
        if (!err) {

            /**
             * Add the query to the status object
             * @type {*}
             */
            var qid = Math.floor(Math.random() * 1000);
            this.status.running[qid] = {
                 query  : query
                ,params : params
            };

            var execute = function(query, params, callback, qid) {

                /**
                 * Execute the query
                 */
                conn.query(query, params, function(err, results) {

                    if (err) {
                        callback(
                            new Error('Mysql query error : ' + err.message)
                        );
                    } else {
                        callback(null, results);
                    }

                    /**
                     * Release the connection
                     */
                    conn.end();

                    /**
                     * Remove the query from the status object
                     */
                    delete this.status.running[qid];

                }.bind(this));

            }.bind(this); execute(query, params, callback, qid);

        } else {
            callback(
                new Error('Mysql connection error : ' + err.message)
            );
        }
    }.bind(this));
};

/**
 * Chooses connection pool based on first word inside the query
 *
 * @param {string} query
 * @return {object}
 */
var choosePool = function(query) {
    var firstWord = query.substring(0, query.indexOf(' ')).toLowerCase();
    if (firstWord === 'select') {
        this.status.querycnt.slave++;
        return this._slavepool;
    }
    this.status.querycnt.master++;
    return this._masterpool;
};

/**
 * Exports
 */
exports = module.exports = Mysql;

==> lemonade-0.1.1.tgz: package/kernel/adaptors/mysql.js <==
/*****************************************************************************
 * Lemonade JS
 * https://lemonadejs.com
 *
 * ----
 *
 * Mysql
 *
 *  Adaptor to connect and execute queries on a Mysql replication farm. Will
 *  create different connections depending on the first word in the query,
 *  directing modifying requests to the master and selects to the slave
 *
 * @param {object} master
 * @param {object} slave
 * @param {string} db
 *
 *****************************************************************************/

function Mysql (master, slave, db) {

    this._masterpool = adaptor.createPool({

         host     : master.host
        ,port     : master.port
        ,user     : master.user
        ,password : master.pass
        ,database : db

        ,connectionLimit : 10
    });

    if (slave) {
        this._slavepool = adaptor.createPool({

             host     : slave.host
            ,port     : slave.port
            ,user     : slave.user
            ,password : slave.pass
            ,database : db

            ,connectionLimit : 10
        });
    } else {
        this._slavepool = this._masterpool;
        this.status.singlepool = true;
    }
}

/**
 * Module dependencies
 */
var adaptor = require('mysql');

/**
 * Status object
 */
Mysql.prototype.status = {
    querycnt : {
         master : 0
        ,slave  : 0
    }
    ,running : {}
    ,singlepool : false
};

/**
 * Master connection pool
 */
Mysql.prototype._masterpool;

/**
 * Slave connection pool
 */
Mysql.prototype._slavepool;

/**
 * Fetches a connection from the pool and runs a query
 * @param {string} query
 * @param {Array} params
 * @param {function} callback
 */
Mysql.prototype.query = function(query, params, callback) {
    this._choosePool(query).getConnection(function(err, conn) {
        if (!err) {
            /**
             * Add the query to the status object
             */
            var qid = Math.floor(Math.random() * 1000);
            this.status.running[qid] = {
                 query  : query
                ,params : params
                ,start  : this.kernel.utility.date(null, 'Y-m-d h:i:s')
            };
            /**
             * Execute the query
             */
            var execute = function(query, params, callback, qid) {
                conn.query(query, params, function(err, results) {

                    if (err) {
                        callback(
                            new Error('Mysql query error : ' + err.message)
                        );
                    } else {
                        callback(null, results);
                    }
                    /**
                     * Release the connection
                     */
                    conn.end();
                    /**
                     * Remove the query from the status object
                     */
                    delete this.status.running[qid];

                }.bind(this));
            }.bind(this); execute(query, params, callback, qid);

        } else {
            callback(
                new Error('Mysql connection error : ' + err.message)
            );
        }
    }.bind(this));
};

/**
 * Waits until all running queries are finished and loop through
 * the active connections in the pool and destroy them
 * @param {function} cb
 */
Mysql.prototype.shutdown = function(cb) {
    var intv = setInterval(function() {
        if (!Object.keys(this.status.running).length) {
            for (var i = 0, arr = [this._masterpool._allConnections, this._slavepool._allConnections]; i < 2; i++) {
                for (var j = 0, max = arr[i].length; j < max; j++) {
                    arr[i][j].destroy();
                }
            }
            clearInterval(intv);
            cb();
        }
    }.bind(this), 1000);
};

/**
 * Chooses connection pool based on first word inside the query
 * @param {string} query
 * @return {object}
 */
Mysql.prototype._choosePool = function(query) {
    var firstWord = query.substring(0, query.indexOf(' ')).toLowerCase();
    if (firstWord === 'select') {
        this.status.querycnt.slave++;
        return this._slavepool;
    }
    this.status.querycnt.master++;
    return this._masterpool;
};

/**
 * Exports
 */
exports = module.exports = Mysql;
